<div class="panel panel-default filter-trigger-container" data-field="{{ field }}" id="{{ field }}-trigger-container">
    <div class="panel-heading" role="tab" id="{{ heading_id }}">
      <h4 class="panel-title">
        <a role="button" data-toggle="collapse" data-parent="#accordion" href="#{{ collapse_id }}" aria-expanded="true" aria-controls="{{ collapse_id }}">
          {{ title }}
        </a>
      </h4>
    </div>
    <div id="{{ collapse_id }}" class="panel-collapse collapse in" role="tabpanel" aria-labelledby="{{ heading_id }}">
      <div class="panel-body">
          {% for item in items %}
              <a class="label label-success filter-trigger" href="?{{ field }}={{ item }}" data-active="true" data-field="{{ field }}" data-value="{{ item }}">{{ item}}</a>
          {% endfor %}
      </div>
    </div>
  </div>